package com.chinacreator.quartz.directed.config;


import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.quartz.QuartzDataSource;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

@Configuration
@MapperScan(basePackages = "com.chinacreator.quartz.directed.*.dao.mydba", sqlSessionTemplateRef = "mydbaSessionTemplate")
public class MydbaConfig {
    @Bean(name = "mydba")
    @QuartzDataSource
    @ConfigurationProperties(prefix = "spring.datasource.mydba")
    @Primary
    public DataSource mydba() {
        return DataSourceBuilder.create().build();
    }

    @Bean(name = "mydbaSqlSessionFactory")
    @Primary
    public SqlSessionFactory mydbaSqlSessionFactory(@Qualifier("mydba") DataSource dataSource) throws Exception {
        SqlSessionFactoryBean factoryBean = new SqlSessionFactoryBean();
        factoryBean.setDataSource(dataSource);
        factoryBean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:mapping/mydba/*.xml"));
        return factoryBean.getObject();
    }

    @Bean(name = "mydbaTransactionManager")
    @Primary
    public DataSourceTransactionManager mydbaTransactionManager(@Qualifier("mydba") DataSource dataSource) {
        return new DataSourceTransactionManager(dataSource);
    }

    @Bean(name = "mydbaSessionTemplate")
    @Primary
    public SqlSessionTemplate mydbaSessionTemplate(@Qualifier("mydbaSqlSessionFactory") SqlSessionFactory sqlSessionFactory) throws Exception {
        SqlSessionTemplate template = new SqlSessionTemplate(sqlSessionFactory); // 使用上面配置的Factory
        return template;
    }
}
