package com.chinacreator.quartz.directed.util;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * @ClassName: DateUtil
 * @Description: 日期工具类
 */
public class DateUtil {
    public static final String YYYYMM = "yyyyMM";
    public static final String YYYYMMDD = "yyyyMMdd";
    public static final String YYYYMMDDHH = "yyyyMMddHH";
    public static final String YYYY_MM_DD = "yyyy-MM-dd";
    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    public static final String YYYY_MM_DD_HH_MM = "yyyy-MM-dd HH:mm";
    public static final String YYYY_MM_DD_HH_MM_SSS = "yyyy-MM-dd HH:mm:ss.SSS";
    public static final String YYYYMMDDHHMMSSS = "yyyyMMddHHmmssSSS";
    public static final String YYYYMMDDHHMMss = "yyyyMMddHHmmss";

    /**
     * 按指定格式将字符串转换为 java.sql.Timestamp
     *
     * @param dateStr
     * @param format
     * @return
     */
    public static Timestamp to_timestamp(String dateStr, String format) throws Exception {
        DateFormat df = new SimpleDateFormat(format);
        Date date = df.parse(dateStr);
        Timestamp d = new Timestamp(date.getTime());
        return d;
    }


    /**
     * 获取当前日期按指定格式显示
     *
     * @param //format(所需指定格式，例如:yyyyMMdd)
     * @return
     */
    public static String NowTime() {
        return new SimpleDateFormat(DateUtil.YYYYMMDD).format(new Date());
    }
    public static String NowTimeSSS() {
        return new SimpleDateFormat(DateUtil.YYYY_MM_DD_HH_MM_SSS).format(new Date());
    }
    public static String NowTimeSS() {
        return new SimpleDateFormat(DateUtil.YYYY_MM_DD_HH_MM_SS).format(new Date());
    }


    /**
     * 获取时间类型
     *
     * @param date
     * @param dateFormat
     * @return
     */
    public static String getDateTimeString(Date date, String dateFormat) {
        return new SimpleDateFormat(dateFormat).format(date);
    }


    /**
     * String转成次格式的Date
     *
     * @param strDate
     * @return
     * @throws Exception
     */
    public static Date StrToDate(String strDate, String format) throws Exception {

        SimpleDateFormat sdf = new SimpleDateFormat(format);
        Date date = sdf.parse(strDate);
        return date;
    }

    /**
     * 获取当前毫秒数
     */
    public static long GetNowDateTime() {
        return (new Date()).getTime();
    }


    /**
     * 获取明天的日期
     *
     * @return
     * @author LiMingyu
     */
    public static String tomorrowDateEn2() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DAY_OF_MONTH, 1);
        return new SimpleDateFormat("yyyyMMdd").format(calendar.getTime());
    }

    public static String tomorrowDateTimeMi(int mis) {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MINUTE, mis);
        return new SimpleDateFormat(YYYY_MM_DD_HH_MM_SS).format(calendar.getTime());
    }

    public static String lastDayHouse(int house,String format) {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.HOUR_OF_DAY, house);
        return new SimpleDateFormat(format).format(calendar.getTime());
    }


    /**
     * 获取某年某月第一天日期
     *
     * @param year   年
     * @param month  月份
     * @param format 显示格式
     * @return
     * @author LiMingyu
     */
    public static String getCurrMonthFirst(int year, int month, String format) {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, year);
        cal.set(Calendar.MONTH, month - 1);
        cal.set(Calendar.DAY_OF_MONTH, cal.getMinimum(Calendar.DATE));
        return new SimpleDateFormat(format).format(cal.getTime());
    }

    /**
     * 获取某年某月最后一天日期
     *
     * @param year   年
     * @param month  月份
     * @param format 显示格式
     * @return
     * @author LiMingyu
     */
    public static String getCurrMonthLast(int year, int month, String format) {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.YEAR, year);
        cal.set(Calendar.MONTH, month - 1);
        cal.set(Calendar.DAY_OF_MONTH, 1);
        int value = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
        cal.set(Calendar.DAY_OF_MONTH, value);
        return new SimpleDateFormat(format).format(cal.getTime());
    }


    /**
     * 获取当前月向前X个月的list
     *
     * @param count 向前推的月份数量
     * @return
     * @author LiMingyu
     */
    public static List<Map<String, Object>> getMonthFromPerMonth(int count) {
        List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
        for (int i = 1; i < count + 1; i++) {
            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.MONTH, -i);
            Map<String, Object> map = new HashMap<String, Object>();
            int year = cal.get(Calendar.YEAR);
            int month = (cal.get(Calendar.MONTH) + 1);
            map.put("formatStr", year + "年" + month + "月");
            map.put("year", year);
            map.put("month", month);
            map.put("yyyyMM", String.valueOf(year) + (month < 10 ? "0" + month : month));
            list.add(map);
        }
        return list;
    }


    /**
     * 返回时间戳对应的 时间戳字符串
     *
     * @param timestamp
     * @return
     * @author Limingyu
     */
    public static String timstamp2String(Timestamp timestamp) {
        if (timestamp == null)
            return "";
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");// 定义格式，不显示毫秒
        String str = df.format(timestamp);
        if (str != null && str.length() > 10)
            str = str.substring(0, 10);
        return str;
    }

    /**
     * 返回时间戳对应的日期字符串
     *
     * @param timestamp
     * @return
     * @author LiMingyu
     */
    public static String timstamp3String(Timestamp timestamp) {
        if (timestamp == null)
            return "";
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");// 定义格式，不显示毫秒
        String str = df.format(timestamp);
        if (str != null && str.length() > 10)
            str = str.substring(0, 10);
        return str;
    }

    /**
     * 返回时间戳字符串对应的时间戳
     *
     * @param timestamp
     * @return
     * @throws Exception
     * @author LiMingyu
     */
    public static Timestamp string2Datestamp(String timestamp) {
        if (timestamp == null || timestamp.length() < 10)
            return null;
        Timestamp ts = Timestamp.valueOf(timestamp.trim() + " 00:00:00");
        return ts;
    }

    public static String timeLongString(long longTime, String dateFormat) {
        Date date = new Date(longTime);
        return new SimpleDateFormat(dateFormat).format(date);
    }

    /**
     * 格式化日期串
     *
     * @return
     * @author LiMingyu
     */
    public static String getFormatDate(Date date, String formatStr) {
        if (formatStr == null || null == date) {
            return "";
        } else {
            return new SimpleDateFormat(formatStr).format(date);
        }
    }

    /**
     * 上n月当天
     * @param lastnum
     * @param date
     * @return
     */
    public static String getLastMothDate(int lastnum,int datenum,String date) {
        SimpleDateFormat datef = new SimpleDateFormat("yyyy-MM-dd");
        Calendar cal = Calendar.getInstance();
        try {
            cal.setTime(datef.parse(date));
            cal.add(Calendar.MONTH, lastnum);
            cal.set(Calendar.DATE,1);
            cal.add(Calendar.DATE, datenum);
            Date beginTime = cal.getTime();
            return datef.format(beginTime);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    public static String getHousnext(int lastnum) {
        Calendar cal = Calendar.getInstance();
        SimpleDateFormat datef = new SimpleDateFormat("yyyy-MM-dd HH");
        // 当前时间前几个小时或者后几个小时
        cal.set(Calendar.HOUR_OF_DAY, cal.get(Calendar.HOUR_OF_DAY) + lastnum);
        Date beginTime = cal.getTime();
        return datef.format(beginTime);
    }

    /**
     * 当前月份
     */
    public static String getCurrentDateMonth_(String format) {
        // e.g. format="yyyyMM";
        return getFormatDate(new Date(), format);
    }

    /**
     * 获取当月的首日和末日 例如： 2014-09-01 2014-09-30
     */
    public static String getMothFday() {
        Calendar cal = Calendar.getInstance();
        SimpleDateFormat datef = new SimpleDateFormat("yyyy-MM-dd");
        // 当前月的第一天
        cal.set(GregorianCalendar.DAY_OF_MONTH, 1);
        Date beginTime = cal.getTime();
        return datef.format(beginTime);
    }

    /**
	 * 获取次月初月份
	 */
	public static String getLastMonth(int num, String format) {
		Calendar c = Calendar.getInstance();
		c.add(Calendar.MONTH, num);
		return new SimpleDateFormat(format).format(c.getTime());
	}


    /**
     * 获取当月的首日和末日 例如： 2014-09-01 2014-09-30
     */
    public static String getMothFday(String date) {
        SimpleDateFormat datef = new SimpleDateFormat("yyyy-MM-dd");
        Calendar cal = Calendar.getInstance();
        try {
            cal.setTime(datef.parse(date));
            cal.set(GregorianCalendar.DAY_OF_MONTH, 1);// 当前月的第一天
            Date beginTime = cal.getTime();
            return datef.format(beginTime);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 当月最后一天
     * @return
     */
    public static String getMothLday() {
        Calendar cal = Calendar.getInstance();
        SimpleDateFormat datef = new SimpleDateFormat("yyyy-MM-dd");
        // 当前月的最后一天
        cal.set(Calendar.DATE, 1);
        cal.roll(Calendar.DATE, -1);
        Date endTime = cal.getTime();
        return datef.format(endTime);
    }

    public static String getMothLday(String date) {
        SimpleDateFormat datef = new SimpleDateFormat("yyyy-MM-dd");
        Calendar cal = Calendar.getInstance();
        try {
            cal.setTime(datef.parse(date));
            cal.set(Calendar.DATE, 1);// 当前月的最后一天
            cal.roll(Calendar.DATE, -1);
            Date endTime = cal.getTime();
            return datef.format(endTime);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 传入时间戳 返回YYYYMM
     *
     * @param timestamp
     * @return
     * @author LiMingyu
     */
    public static String timstamp4String(Timestamp timestamp) {
        if (timestamp == null)
            return "";
        SimpleDateFormat df = new SimpleDateFormat("yyyyMM");// 定义格式，不显示毫秒
        String str = df.format(timestamp);
        if (str != null && str.length() > 10)
            str = str.substring(0, 10);
        return str;
    }

    /**
     * 获取当前时间戳
     *
     * @return
     * @author LiMingyu
     */
    public static Timestamp getTimestamp() {
        return new Timestamp(System.currentTimeMillis());

    }


    /**
     * 计算两个参数时间相差多久
     *
     * @param stardate
     * @param endtime
     * @return
     */
    public static long mathday(Date stardate, Date endtime) {

        if (stardate == null && endtime == null) {
            return -1L;
        }
        return (endtime.getTime() - stardate.getTime());
    }


    /**
     * 获取当前月的上num个月
     *
     * @return
     */
    public static String GetLastMonth(int num) {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, num);
        return new SimpleDateFormat("yyyy-MM").format(c.getTime());
    }

    /**
     * 获取当前时间前后n分钟
     *
     * @return
     */
    public static String GetLastMINUTE(int num) {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MINUTE, num);
        return new SimpleDateFormat(DateUtil.YYYY_MM_DD_HH_MM).format(c.getTime());
    }

    public static boolean isEffectiveDate(String now, String start, String endt) {
        Date nowTime = new Date();
        try {
            if (!(now == null || "".equals(now))) {
                nowTime = StrToDate(now, YYYY_MM_DD_HH_MM_SS);
            }
            Date startTime = StrToDate(start, YYYY_MM_DD_HH_MM_SS);
            Date endTime = StrToDate(endt, YYYY_MM_DD_HH_MM_SS);
            if (nowTime.getTime() == startTime.getTime()
                    || nowTime.getTime() == endTime.getTime()) {
                return true;
            }

            Calendar date = Calendar.getInstance();
            date.setTime(nowTime);
            Calendar begin = Calendar.getInstance();
            begin.setTime(startTime);
            Calendar end = Calendar.getInstance();
            end.setTime(endTime);
            if (date.after(begin) && date.before(end)) {
                return true;
            } else {
                return false;
            }
        }catch (Exception e){
            e.printStackTrace();
            return false;
        }
    }

    /**
     * 前后n天
     * @param lastnum
     * @return
     */
    public static String getLastDate(int lastnum){
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Calendar c = Calendar.getInstance();
        c.add(Calendar.DATE, lastnum);
        Date start = c.getTime();
        String qyt= format.format(start);
        return qyt;
    }

    /**
     * 前后n天自定义格式
     * @param lastnum
     * @param dateformat
     * @return
     */
    public static String getLastDate(int lastnum,String dateformat){
        SimpleDateFormat format = new SimpleDateFormat(dateformat);
        Calendar c = Calendar.getInstance();
        c.add(Calendar.DATE, lastnum);
        Date start = c.getTime();
        String qyt= format.format(start);
        return qyt;
    }

    public static void main(String[] args) {
        try {
//            Map<String,String> maps=new HashMap<>();
//            maps.put("begdate",DateUtil.getMothFday(DateUtil.GetLastMonth(-1)+"-01")+" 00:00:00");
//            maps.put("enddate",DateUtil.getMothLday(DateUtil.GetLastMonth(-1)+"-01")+" 23:59:59");
//            System.out.println(JSON.toJSONString(maps));
//            System.out.println("--------------"+DateUtil.tomorrowDateTimeMi(-5));
//            System.out.println( DateUtil.getLastDate(-1,DateUtil.YYYYMMDD));
//            System.out.println(DateUtil.tomorrowDateTimeMi(-5));
//            System.out.println(DateUtil.getLastDate(-1,DateUtil.YYYY_MM_DD)+" 23:59:59");
//
//            System.out.println("上个月的月份---" + GetLastMonth(-1));
//            System.out.println("当前的月份-----" + getCurrentDateMonth_("yyyy-MM"));
//            System.out.println("2021-05-31 20:00:00".substring(0,7));
//            System.out.println(NowTimeSS());
            System.out.println(DateUtil.format(DateUtil.StrToDate(DateUtil.getMothLday()+" 23:59:59","yyyy-MM-dd HH:mm:ss"), "yyyyMMddHHmmss"));
            System.out.println("当月最后一天---" + getHousnext(1)+":59:59");
            System.out.println(DateUtil.getLastMothDate(1, -3, DateUtil.getFormatDate(new Date(), DateUtil.YYYY_MM_DD)));
            System.out.println(GetLastMINUTE(-3));
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    public static String format(Date date, String pattern) {
		DateFormat format = null;
		try {
			format = new SimpleDateFormat(pattern);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			format = new SimpleDateFormat("yyMMdd");
		}

		return format.format(date);
	}


    public static String getLastDateyyMMdd(int lastnum,String datefor){
        SimpleDateFormat format = new SimpleDateFormat(datefor);
        Calendar c = Calendar.getInstance();
        c.add(Calendar.DATE, lastnum);
        Date start = c.getTime();
        String qyt= format.format(start);
        return qyt;
    }

    /**
     * 获取某个月份的最后的一天
     *
     * @param date
     * @return
     */
    public static Date getLastDateOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        return calendar.getTime();
    }

    /**
     * 获取当前日期的yyyyMM显示
     *
     * @return
     * @author LiMingyu
     */
    public static String nowDateyyyyMM() {
        return new SimpleDateFormat("yyyyMM").format(new Date());
    }
}
